/*
 * File:   main.c
 * Authors: Cory Baxes and Tim Stroup
 * Project: Green Machine (Chlamydomonas Reinhardtii and/or Chlorella) by Chatfield Senior High School
 *
 * Created on October 6, 2014, 12:53 PM
 * Modified on April 1, 2015, 1:13 PM
 */

#include <nesi.h>
#include <string.h>


int lightsOn = 0;
int x = 0;
void checkSystems(void)
{
    if (powerDriverA.isFailure())
    {
        dataLog.add("Error with Power Driver A on initialization.", 900);
    }
    if (!camera.isOk())
    {
        dataLog.add("Error with camera on initialization.", 901);
    }
    /*if (!co2.status())
    {
        dataLog.add("Error with Hydrogen sensor on initialization.", 31);
    }*/
}
void sunrise(void)
{
    x = 0;
    while (x < 100)
    {
        x++;
        ledR.dutycycle(x);
        ledB.dutycycle(x);
        wait(72000);
    }

}
void sunrise_older (void)
{ do {
    ledR.dutycycle(10);
    ledB.dutycycle(10);
    wait(8000);
    x = 1;
} while(x == 0);
do {
    ledR.dutycycle(20);
    ledB.dutycycle(20);
    wait(8000);
    x = 2;
} while(x == 1);
do {
    ledR.dutycycle(30);
    ledB.dutycycle(30);
    wait(8000);
    x = 3;
} while(x == 2);
do {
    ledR.dutycycle(40);
    ledB.dutycycle(40);
    wait(8000);
    x = 4;
} while(x == 3);
do {
    ledR.dutycycle(50);
    ledB.dutycycle(50);
    wait(8000);
    x = 5;
} while(x == 4);
do {
    ledR.dutycycle(60);
    ledB.dutycycle(60);
    wait(8000);
    x = 6;
} while(x == 5);

do {
    ledR.dutycycle(70);
    ledB.dutycycle(70);
    wait(8000);
    x = 7;
} while(x == 6);

do {
    ledR.dutycycle(80);
    ledB.dutycycle(80);
    wait(8000);
    x = 8;
} while(x == 7);
do {
    ledR.dutycycle(100);
    ledB.dutycycle(100);
    wait(8000);
    x =11;
} while(x == 8);
}

void mosphetDrain (void)
{
    powerDriverB.on();
}
void startAgitation(void)
{
    // turn the Hydrogen sensor off to save power for motor
    //co2.off();
    powerDriverA.on();
  
}

void stopAgitation(void)
{
    powerDriverA.off();
    // turn the Hydrogen sensor back on
    //co2.on();
}

void cycleLights(void)
{
    
    
        ledR.dutycycle(100);
        ledB.dutycycle(100);
        lightsOn = 1;
    
   
}

void logHydrogen(void)
{
    // turn Hydrogen sensor back on for reading
    //co2.on();

    // read Hydrogen sensor value. 10 readings 100ms apart
    unsigned hydrogenValue = co2.get(10,100);

    // turn off Hydrogen sensor during power intensive operations
    //co2.off();

    // log the Hydrogen sensor value
    dataLog.add("Hydrogen sensor value = ", hydrogenValue);

}

Boolean takePicture(void)
{
    char filename[32] = {0};
    sprintf(filename, "%s.jpg", dateTime.getStamp());

    // the ':' character is an invalid character, so it needs to be changed
    String temp = filename;
    while(*temp)
    {
        if(*temp == ':')
            *temp = '.';
        ++temp;
    }
    camera.on();
    wait(1000);
    return camera.getPix(filename);
    camera.off();
}
void storeLightSensorQ1(void)
{
    Uint valQ1 = resistiveSensors.getQ1(10,100);
    char time[128] = {""};
    sprintf(time, "Time: %s", dateTime.getStamp());
    dataLog.add(time, 0);
    dataLog.add("Quadrant 1 = ", valQ1);
}

void storeValues(void)
{
   
    {
    usb.disconnect();
    // log the start of the data cycle
    dataLog.add("Data cycle readings at ", dateTime.getStamp());

    // turn off Hydrogen sensor during power intensive operations
    //co2.off();

     if (lightsOn == 1)
     {
        if(takePicture()) // log if error occurred while attempting to take picture
        {
            char temp[128] = {0};
            sprintf(temp,"Error taking picture at time: %s ", dateTime.getStamp());
            dataLog.add(temp, 48);
        }
     }
     else
     {
         dataLog.add("No picture taken due to Light Cycle at ", dateTime.getStamp());
     }
     //wait(2000);
     //logHydrogen();
   
     usb.connect();

     
    }
   
     // turn Hydrogen sensor back on
     //co2.on();
}

int main_camera(void)
{
    // Initializes NESI modules
    nesi.init();

    usb.connect();
    wait(1000);

    Keystroke key = KEYSTROKE_NONE;

    while(1)
    {
        // get keystroke
        key = button.getStroke();

        if(key == KEYSTROKE_SINGLE) // if button is tapped once
        {
            // disable button keystroke detection
            button.disableStroke();

            // disconnect USB SD card (in case routine writes to SD memory)
            usb.eject();

            char temp[128] = {0};
            if(takePicture()) // log if error occurred while attempting to take picture
                sprintf(temp,"Error taking picture at time: %s ", dateTime.getStamp());
            else
                sprintf(temp,"Successfully took picture at time: %s ", dateTime.getStamp());

            dataLog.add(temp, 0x1010);


            // reconnect USB SD card (to read files)
            usb.connect();

            // enable button keystroke detection
            button.enableStroke();

          while(1)
           {
                usb.process();
            }
        }

    }

    return 0;
}

int main_Debug(void)
{
    // initialize all modules
    nesi.init();

    DateAndTime timeTemp;
    DateAndTime lastTime;

     checkSystems();
    
  /*   do{
     sunrise();
    } while(x < 5);
    x = 7;*/
     usb.connect();


     

    // CYCLES
   while(1)
    {
        timeTemp = dateTime.get();

        // ORIGINAL CYCLE VALUES
        //        LOG DATA: if((timeTemp.hour % 1) == 0)
        // AGITATION CYCLE: if((timeTemp.minute % 30) == 0)
        //     LIGHT CYCLE: if((timeTemp.hour % 12) == 0)

        // Lights on all the time
        {
            cycleLights();
        }
       
        {
            mosphetDrain();
        }

        // log data every hour
        if((timeTemp.hour % 1) == 0 && timeTemp.minute == 0 && (timeTemp.second == 0 || (lastTime.second == 59 && timeTemp.second != 59)))
        {
            
            storeValues();

        }
        
    if((timeTemp.minute % 30) == 0 && (timeTemp.second == 0 || (lastTime.second == 59 && timeTemp.second != 59)))
       {
           startAgitation();
          //   let it agitate for 60 seconds
            // CHANGE TO 60 SECONDS
           wait(60000);
            stopAgitation();
        }
      
     
  
        usb.process();
        lastTime = timeTemp;

   }
 

    return 0;
}

//ACTUAL MAIN FOR PROGRAM
int main(void)
{
    // initialize all modules
    nesi.init();

    // Define timestamp variable
    DateAndTime timeTemp;
    DateAndTime lastTime;

    // checks Camera, Power Driver B, and hydrogen sensor for problems
    checkSystems();
   /*do{
     sunrise();
    } while(x < 10);
    x = 10;*/
    sunrise();
    // start the Hydrogen sensor
    //co2.on();

    // connects to USB host
    usb.connect();

    // CYCLES
    while(1)
    {
        timeTemp = dateTime.get();

        // ORIGINAL CYCLE VALUES
        //        LOG DATA: if((timeTemp.hour % 1) == 0)
        // AGITATION CYCLE: if((timeTemp.minute % 30) == 0)
        //     LIGHT CYCLE: if((timeTemp.hour % 12) == 0)

        // light cycle every 12 hours
       
        {
            cycleLights();
        }
// uncomment the store values function for real experiment
        // log data every hour
        /*if((timeTemp.hour % 1) == 0 && timeTemp.minute == 0 && (timeTemp.second == 0 || (lastTime.second == 59 && timeTemp.second != 59)))
        {
           storeValues();
        }*/

        if((timeTemp.minute % 1) == 0 && (timeTemp.second == 0 || (lastTime.second == 59 && timeTemp.second != 59)))
        {
           storeValues();
        }

        // agitation cycle every 30 minutes
        if((timeTemp.minute %30) == 0 && (timeTemp.second == 0 || (lastTime.second == 59 && timeTemp.second != 59)))
        {
            startAgitation();
            // let it agitate for 60 seconds
            wait(60000);
            stopAgitation();
        }
       
        powerDriverB.on();

        usb.process();
        lastTime = timeTemp;
    }

    return 0;
}